#include <stdio.h>
#include <stdlib.h> 

#include "tvsserver_base.h"
#include "tvs_redefine_api_names.h"
#include "http_c_example_init.h"
#include "http_c_example_handle.h"

int __stdcall _HandleConnection(ProtocolSession *psess, ClientSession *sess) {
    int len, reqErr;
	char *root=0, *urlBuf=0, *strBuf=0, *tmpPtr;
	ResponseHeader respHead={0};
	HTTPSession* httpRequest;
	HANDLE hfile;
 
	#ifdef _WIN64
#else
	//in 32-bit mode, the long long's are pointers, create memory
	respHead.contentLength = malloc(sizeof(long long));
	respHead.contentRangeOffset = malloc(sizeof(long long));
	respHead.contentRangeTotalLength = malloc(sizeof(long long));
	respHead.lastModified = malloc(sizeof(long long));
	respHead.expires = malloc(sizeof(long long));

	//Set them on zero
	*respHead.contentLength = 0;
	*respHead.contentRangeOffset = 0;
	*respHead.contentRangeTotalLength = 0;
	*respHead.lastModified = 0;
	*respHead.expires = 0;
#endif

    if(psess==http_psess) {
        len=100;

		//Receive and parse the HTTP command
		httpRequest = RecvAndParseHTTP(psess, sess, 0, 0,RPF_CHECK_VALID_METHOD, &reqErr);
		if(httpRequest==0) return 0; //return if the HTTP-command hasn't been received

		//Convert escaped characters to bytes and delete double slashes
		ConvertEscapedCharactersToBytes((char*) httpRequest->urlBuffer, 0);
		DeleteDoubleSlashesInUrl((char*) httpRequest->urlBuffer);

		//solve the parent and current directory problem
		if(!DecodeURL((char*) httpRequest->urlBuffer)){
			//return if the requested URL tries to get under the root
			FreeHTTPSession(httpRequest); 
			return 0;
		}

	    //get the length of the server path
        len = 0;
        GetPluginData(psess, 0, (ClientSession*) sess, "opt1", (char*) 0, &len, 0);
 
        //allocate memory to receive the memory path
        root = (char*) malloc(len+ 1);
		memset(root, 0, len+1);


        //reveice the server path
        if(!GetPluginData(psess, 0, (ClientSession*) sess, "opt1", root, &len, 0)){
			//return if the requested URL tries to get under the root
			if(root) free(root); root=0;
			FreeHTTPSession(httpRequest); 
			return 0;
		}

		//urlBuf => root + url
		urlBuf = (char*) malloc(strlen((char*) httpRequest->urlBuffer) + len + 1);
		sprintf(urlBuf,"%s%s", root, httpRequest->urlBuffer);

		//Open the file
		hfile = CreateFileA(urlBuf,\
			GENERIC_READ,\
			FILE_SHARE_READ,\
			0,\
			OPEN_EXISTING,\
			0,\
			0);

		//send 404 error if the file does not exist
		if(hfile==INVALID_HANDLE_VALUE) {
			respHead.statusCode = STATUS_404; //404 Not Found status
			respHead.contentType = TEXT_PLAIN; //Content-Type = TEXT/PLAIN
			respHead.flags = FRH_CONNECTION_CLOSE; //Set "Connection: close"
			
			len = strlen("The URL '' was not found.") + strlen((char*) httpRequest->urlBuffer)+1;

			//Fill in the contentLength (-1 because we dont want to send the zero byte)
			#ifdef _WIN64
				respHead.contentLength = len - 1;
			#else
				*respHead.contentLength = len - 1;
			#endif

			//Send HTTP header
			SendHTTPHeader(psess, sess, &respHead, 5*1000);

			//Create content
			strBuf=(char*) malloc(len);
			sprintf(strBuf, "The URL '%s' was not found.", httpRequest->urlBuffer);

			//Send content
			len--; //Dont send the zero byte

			SendData(psess, sess, 0, 5*1000, strBuf, &len);

			if(strBuf) free(strBuf); strBuf=0;
			if(urlBuf) free(urlBuf); urlBuf=0;
			if(root) free(root); root=0;
			//free resources allocated in the HTTPSession structure
			FreeHTTPSession(httpRequest); 

			return 1;
		}


		//find extension of the requested file
		tmpPtr = urlBuf;
		len = strlen(tmpPtr);
		tmpPtr += len;

		respHead.contentType = UNKNOWN_BINARY; //Content-Type = TEXT/PLAIN

		while(len!=0) {
			tmpPtr--;
			len--;
			if(*tmpPtr=='.') {
				respHead.contentTypeExtension = tmpPtr; //the function SendHTTPHeader should get the Content-type from the extension
				respHead.flags |= FRH_USE_CONTENT_TYPE_EXTENSION; //set flag FRH_USE_CONTENT_TYPE_EXTENSION
				break;
			}
		}

		//fill in the file size
		#ifdef _WIN64
			GetFileSizeEx(hfile, (PLARGE_INTEGER) &respHead.contentLength);
		#else
			GetFileSizeEx(hfile, (PLARGE_INTEGER) respHead.contentLength);
		#endif

		respHead.statusCode = STATUS_200; //200 OK
		respHead.flags |= FRH_CONNECTION_CLOSE; //Set "Connection: close"

		//Send HTTP header
		SendHTTPHeader(psess, sess, &respHead, 5*1000);

		//Send the file
		SendFile(psess, sess, hfile, 4096, 0, 0, 5*1000);

		//close file handle
		CloseHandle(hfile);

			if(urlBuf) free(urlBuf); urlBuf=0;
			if(root) free(root); root=0;
		//free resources allocated in the HTTPSession structure
		FreeHTTPSession(httpRequest); 
        return 1; //we are done, return so the other protocols can close the connection
    }
    return 0;
}