
//TV's server version 3.00
//Copyright (c) 2014, Tycho Veltmeijer.
//All rights reserved.

#ifndef __TVSSERVER_BASE_H__
#define __TVSSERVER_BASE_H__

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <winsock2.h>
#include <ws2tcpip.h>

#ifdef _WIN64
	#define long_int long long
#else
	#define long_int int
#endif

#define false 0
#define true 1

//--Flags--
//internal listensession state
#define LSESS_STATE_LOADING 0
#define LSESS_STATE_ERROR 1
#define LSESS_STATE_LOADED 2
#define LSESS_STATE_UNLOADING 3

//ListenSessionIsCreated flags
#define LCCF_LSESS_UNLOADED 0x1

//status flags
#define PT_STARTING 0
#define PT_PROTOCOLINFO_SET 1
#define PT_RUNNING 2


//ControlPanelOption structure flags
#define CPO_PRIV_PROTOCOL_OPTION 0x1
#define CPO_PRIV_LISTENSESSION_OPTION 0x2
#define CPO_PRIV_READABLE 0x4
#define CPO_PRIV_WRITEABLE 08

//--or--
#define CPO_SHOW_IN_MENU 0x10

//--or--
#define CPO_GEN_PROTOCOL_OPTION 0x20
#define CPO_GEN_LISTENSESSION_OPTION 0x200
#define CPO_GEN_READONLY 0x4
#define CPO_GEN_LISTENSESSION_READONLY 0x8

//--and--
#define CPO_OPTIONNAME_IS_UNICODE 0x40
#define GDPD_NAME_IS_INTEGER 0x80
#define CPO_DESCRIPTION_IS_UNICODE 0x100

//---------

//BindToProtocolCallBack function flags
#define BTPC_RECOVERED 0x1

//---------

//GetClientSessionInfo function flags
#define CSI_CLIENT_ADDRESS 0x0
#define CSI_CLIENT_PORT 1
#define CSI_LOCAL_ADDRESS 2
#define CSI_LOCAL_PORT 3

//--and--
#define CSI_UNICODE_STRING 0x0
#define CSI_ASCII_STRING 0x10000
#define CSI_RAW_DATA 0x20000

//---------

//GetPluginData & SavePluginData function flags
#define GDPD_IS_LIST 0x1
#define GDPD_NO_COPY_LIST 0x2
#define GDPD_MEMORY_ONLY 0x4
//#define GDPD_NAME_IS_INTEGER 0x80 --see ControlPanelOption structure flags
#define GDPD_RESET_DATA 0x8

//---------

//GetSettingsHeaderSize & SendSettingsHeader function flags
#define HEADER_TOP 0x1
#define HEADER_BOTTOM 0x2

//---------

//GetSupportedLanguages function flags
#define L_US_ENGLISH 0

//---------

//RecvData & SendData  function flags
#define RSF_RETURN_ON_COMPLETE 0x0
#define RSF_RETURN_ON_ERROR 0x1
#define RSF_PEEK_DATA 0x2
#define RSF_RETURN_ON_FIRST_SUCCESSFUL_ATTEMPT 0x4

//RecvData & SendData return values
#define CE_NOERR 0x0
#define CE_NO_BUFFER_AVAILABLE 0x1
#define CE_CONNECTION_CLOSED 0x2
#define CE_UNKNOWN_ERROR 0x3
#define CE_TIMEOUT 0x4
//#define CE_TIMEOUT_NODATA 0x5

//---------

//SetProtocolInformation & BindStruct function flags en types
#define PT_RECEIVER 0
#define PT_MANIPULATOR 1
#define PT_LABORER 2
#define PT_NO_PROTOCOL 3

#define PT_HIGHEST_FLAG PT_NO_PROTOCOL

#define PT_CONTROLPANEL 0x10000
#define PT_SUPPORT_PSIGN 0x20000

#define PT_SYSTEM_DEFAULT_CHARSET 0x0
#define PT_ANSI_CHARSET 0x40000
#define PT_MAC_CHARSET 0x80000
#define PT_UTF_8_CHARSET 0x100000

#define V_TV_SERVER_1 0
//#define INTERNAL_V_TV_SERVER_1_C_PROTOCOL 0x10000

//---------

//BindStruct funciton return values
#define BE_NOERR 0
#define BE_PROTOCOL_LOADING 1
#define BE_NOT_LOADED 2
#define BE_NOT_SUPPORTED_PROTOCOL 3

//---------

//UserValueOption structure types
#define TD_ASCII_STRING 0
#define TD_UNICODE_STRING 1
#define TD_BOOLEAN 2
#define TD_CHAR 3
#define TD_SHORT_INT 4
#define TD_INTEGER 5
#define TD_LARGE_INTEGER 6
#define TD_DATE 7
#define TD_TIME 8
#define TD_DATETIME 9
#define TD_PROGRESS 10
#define TD_LIST 11


//--Errors
#define NO_ERROR 0
#define PT_WRONG_TYPE 1
#define PSESS_NOT_VALID 2
#define VERSION_UNSUPPORTED 3
#define INVALID_POINTER 4
#define UVO_WRONG_TYPE 5
#define CORRUPT_SIZE 6
#define LSESS_NOT_VALID 7

//ResponseHeader flags
#define FRH_DONT_SEND_TIMEDATE 0x1
#define FRH_USE_CONTENT_TYPE_EXTENSION 0x2
#define FRH_SEND_CONTENT_RANGE 0x4
#define FRH_FORCE_ZERO_CONTENT_LENGTH 0x8
#define FRH_SEND_LAST_MODIFIED 0x10
#define FRH_SEND_EXPIRES 0x20
#define FRH_SEND_AUTHORIZATION_REQUEST 0x40
#define FRH_DONT_END_HEADER 0x80
#define FRH_CONNECTION_CLOSE 0x0
#define FRH_CONNECTION_KEEP_ALIVE 0x100
#define FRH_DONT_ESCAPE_LOCATION 0x200
#define FRH_DONT_SEND_CONTENT_TYPE 0x400

//RecvAndParseHTTP flags
#define RPF_DONT_PARSE 0x1
#define RPF_USE_HEADER_SIZE_LIMIT 0x2
#define RPF_USE_TIME_LIMIT 0x4
#define RPF_CHECK_VALID_METHOD 0x8

//RecvAndParseHTTP error codes
#define RPE_NO_ERROR 0
#define RPE_EXCEEDED_SIZE_LIMIT 1
#define RPE_EXCEEDED_TIME_LIMIT 2
#define RPE_UNKNOWN_METHOD 3
#define RPE_UNKNOWN_PROTOCOL 4
#define RPE_CONNECTION_LOST 5

//EscapeAndSendValue flags
#define REPLACE_SPACE_WITH_ADD_SIGN 0x1
#define SKIP_SPECIAL_CHARS 0x2
#define DONT_REPLACE_PATH_CHARACTERS 0x4

//EscapeHtmlAndSendValue flags
#define REPLACE_SPACE_WITH_NON_BREAKING_SPACE 0x1

//cookie flags
#define CF_SECURE 0x1
#define CF_HTTP_ONLY 0x2
#define CF_DONT_ESCAPE_VALUE 0x4
#define CF_DONT_ESCAPE_PATH 0x8
#define CF_SEND_EXPIRES_DATE 0x10

//AuthorizationOptions flags
#define AF_BASIC_AUTHENTICATION 0x0
#define AF_DIGEST_AUTHENTICATION 0x1

#define AF_QOP_AUTH 0x0
#define AF_QOP_AUTH_INT 0x2

#define AF_STALE_DONT_SEND 0x0
#define AF_STALE_TRUE 0x4
#define AF_STALE_FALSE 0x8

//ConvertEscapedCharacterToBytes flags
#define ETBF_UNESCAPE_ADD_SIGN_AS_SPACE 0x1

//FindListItem flags
#define FLI_CASE_SENSITIVE 0x1
#define FLI_COMPARE_ONLY_ONE_ROW 0x2

//GetAPIVersion flags
#define GAV_LOWEST_SUPPORTED_VERSION 0x1

//ResponseHeader structure Status enum
#define STATUS_100 (int*) 100
#define STATUS_200 (int*) 200
#define STATUS_204 (int*) 204
#define STATUS_206 (int*) 206
#define STATUS_301 (int*) 301
#define STATUS_302 (int*) 302
#define STATUS_304 (int*) 304
#define STATUS_307 (int*) 307
#define STATUS_401 (int*) 401
#define STATUS_403 (int*) 403
#define STATUS_404 (int*) 404
#define STATUS_405 (int*) 405
#define STATUS_408 (int*) 408
#define STATUS_413 (int*) 413
#define STATUS_500 (int*) 500

//ResponseHeader contentType
#define UNKNOWN_BINARY (int*) 0
#define TEXT_PLAIN (int*) 1
#define TEXT_HTML (int*) 2
#define IMAGE_PNG (int*) 3
#define IMAGE_GIF (int*) 4
#define IMAGE_JPEG (int*) 5

//Protocol Initialisation structures 
typedef struct ProtocolSession{
	int size;
	char *protocolName;
	int lastError;
} ProtocolSession;

typedef struct ProtocolData{
	int size;
	ProtocolSession *psess;
	char data[];
} ProtocolData;

typedef struct ClientSession{
	int size;
	int flags;
	struct ProtocolData **data;
} ClientSession;

typedef struct ListenSession{
	int size;
	int flags;
	ProtocolData **data;
} ListenSession;

//Protocol binding structures 

typedef struct BindInitOption{
	char *name;
	char *data;
	int len;
	int flags;
} BindInitOption;


typedef struct BindInitOptions{
	int size;
	int flags;
	BindInitOption listOptions[];
} BindInitOptions;

typedef struct BindStruct{
	char *protocolName;
	int type;
	int errCode;
	struct BindStruct *next;
	struct BindInitOptions *initOptions;
} BindStruct;

//Protocol Listening and handling structures


//Protocol data storage structures

//Control panel structures

typedef struct ProtocolSessionCollection{
	int count;
	int reserved;
	ProtocolSession *psess[];
} ProtocolSessionCollection;

typedef struct InternalListenStructure{
	ProtocolSessionCollection *bindedToHandles;
	void *reserved;
	void *reserved1;
	void *reserved2;
	void *reserved3;
	int status;
	int id;
	void *reserved4;
} InternalListenStructure;

typedef struct UserValueOption{
	int type;
	int defaultValueSize;
	void* defaultValue;
} UserValueOption;

typedef struct ControlPanelOption{
	int size;
	int flags;
	char *name;
	char *optionName;
	char *description;
	char *groupName;
	char *iconURL;
	char *bigIconURL;
	char *menuURL;
	UserValueOption *userValue;
} ControlPanelOption;

typedef struct ControlPanelOptions{
	int size;
	int reserved;
	ControlPanelOption main;
	ControlPanelOption subOptions[];
} ControlPanelOptions;

typedef struct LinkedlistControlPanelOptions{
	struct LinkedlistControlPanelOptions *next;
	ControlPanelOption cpo;
} LinkedlistControlPanelOptions;

typedef struct ProtocolSignature{
	int len;
	char buf[256];
} ProtocolSignature;

typedef struct InternalProtocolStructure{
	struct InternalProtocolStructure *nextProtocol;
	struct ProtocolSession *psess;
	struct ProtocolSession *copyPsess;
	void *reserved;
	void *reserved1;
	struct LinkedlistControlPanelOptions *main;
	struct LinkedlistControlPanelOptions *subOptions;
	int status;
	int type;
	int lowestSupportedVersion;
	int extraSize;
	void *reserved2;
	void *reserved3;
	struct ProtocolSignature *pSign;
	void *reserved4;
} InternalProtocolStructure;

typedef struct CurrentProgress{
	int curPos;
	int minimum;
	int maximum;
} CurrentProgress;

//Multi column list structures

typedef struct Column{
	char *optionName;
	struct UserValueOption *userValue;
} Column;

typedef struct Row{
	int size;
	int reserved;
	struct Row *nextRow;
	void *items[];
} Row;

typedef struct MultiColumnList{
	int size;
	int flags;
	struct Column columns[1];
	struct Row *rows;
} MultiColumnList;

//Time and date structures

typedef struct TIMEDATE{
	int year;
	short month;
	short dayOfTheWeek;
	short day;
	short hour;
	short minutes;
	short seconds;
	short milliseconds;
} TIMEDATE;

//HTTP structures
#ifdef _WIN64
	typedef struct AuthorizationOptions{
		int flags;
		int reserved;
		char *realm;
		char *domain;
		char *nonce;
		char *opaque;
		char *algorithm;
	} AuthorizationOptions;

	typedef struct CookieField{
		struct CookieField *nextCookie;
		int flags;
		int reserved;
		char *name;
		char *value;
		char *domain;
		char *path;
		long long expires;
	} CookieField;

	typedef struct ResponseHeader{
		int flags;
		int reserved;
		int *statusCode;
		int *contentType;
		char *contentTypeExtension;
		long long contentLength;
		long long contentRangeOffset;
		long long contentRangeTotalLength;
		long long lastModified;
		long long expires;
		struct CookieField *cookie;
		AuthorizationOptions requestAuthentication;
		char *location;
	} ResponseHeader;
#else
	typedef struct AuthorizationOptions{
		int flags;
		char *realm;
		char *domain;
		char *nonce;
		char *opaque;
		char *algorithm;
	} AuthorizationOptions;

	typedef struct CookieField{
		struct CookieField *nextCookie;
		int flags;
		char *name;
		char *value;
		char *domain;
		char *path;
		long long *expires;
	} CookieField;

	typedef struct ResponseHeader{
		int flags;
		int reserved;
		int *statusCode;
		int *contentType;
		char *contentTypeExtension;
		long long *contentLength;
		long long *contentRangeOffset;
		long long *contentRangeTotalLength;
		long long *lastModified;
		long long *expires;
		struct CookieField *cookie;
		AuthorizationOptions requestAuthentication;
		char *location;
	} ResponseHeader;
#endif

typedef struct DefaultHTTPVariables{
	char *httpVersion;
	char *method;
	char *url;
	char *host;
	char *date;
	char *range;
	char *accept;
	char *pragma;
	char *cookie;
	char *referer;
	char *ifMatch;
	char *ifRange;
	char *userAgent;
	char *connection;
	char *contentType;
	char *authorization;
	char *cacheControl;
	char *ifNoneMatch;
	char *contentLength;
	char *acceptLanguage;
	char *acceptEncoding;
	char *ifModifiedSince;
	char *ifUnmodifiedSince;
} DefaultHTTPVariables;

typedef struct HTTPSession{
	struct DefaultHTTPVariables *defaultVariables;
	void *otherVariablesStringIndexedCollection;
	void *optionsQuestionmarkStringIndexedCollection;
	void *cookiesStringIndexedCollection;
	void *buffer;
	void *urlBuffer;
	void *cookieBuffer;
	int headerSize;
	int headerRestSize;
	int timeLeft;
} HTTPSession;


//Protocol Initialisation functions 

typedef int __stdcall AcceptProtocol(
	__in	ProtocolSession *psess
);

typedef int __stdcall GetAPIVersion(
	__in	int flags
);

typedef int APIENTRY GetProtocolNames(
	__out	char *buf,
	__inout	int *len
);

typedef int __stdcall SetProtocolInformation(
	__in	ProtocolSession *psess,
	__in	int type,
	__in	ProtocolSignature *pSign,  //changed
	__in	int extraSize,
	__in	int lowestSupportedVersion
);

typedef int __stdcall SetControlPanelOptions(
	__in	ProtocolSession *psess,
	__in	ControlPanelOptions *options
);

typedef int APIENTRY StartProtocol(
	__in	ProtocolSession *psess
);

//Protocol binding functions 

typedef int __stdcall BindToProtocol(
	__in	ProtocolSession *psess,
	__in	BindStruct *bindTo
);

typedef int __stdcall BindToProtocolCallBack(
	__in	ProtocolSession *psess,
	__in	ListenSession *lsess,
	__in	int flags
);

typedef int __stdcall RequestToBindToNextProtocol(
	__in	ProtocolSession *psess,
	__in	ListenSession *lsess
);

typedef int __stdcall PrepareCloseListenSession(
	__in	ProtocolSession *psess,
	__in	ListenSession *lsess
);

//Protocol Listening and handling functions

typedef int __stdcall CallNextHandleConnection(
	__in	ProtocolSession *psess,
	__in	ClientSession *sess
);

typedef int __stdcall CloseClientSession(
	__in	ProtocolSession *psess,
	__in	ClientSession *sess
);

typedef int __stdcall GetClientSessionInfo(
	__in	ProtocolSession *psess,
	__in	ClientSession *sess,
	__in	int type,
	__out	void* buf,
	__inout	int* len
);

typedef int __stdcall HandleConnection(
	__in	ProtocolSession *psess,
	__in	ClientSession *sess
);

typedef int __stdcall RecvData(
	__in	ProtocolSession *psess,
	__in	ClientSession *sess,
	__in	int flags,
	__in	int timeout,
	__out	char *buf,
	__inout	int *len
);

typedef int __stdcall SendData(
	__in	ProtocolSession *psess,
	__in	ClientSession *sess,
	__in	int flags,
	__in	int timeout,
	__in	char *buf,
	__inout	int *len
);

typedef int __stdcall StartListening(
	__in	ProtocolSession *psess,
	__in	ListenSession *lsess,
	__in	ClientSession *sess
);

//Protocol data storage functions

typedef int __stdcall GetPluginData(
	__in	ProtocolSession *psess,
	__in	ListenSession *lsess,
	__in	ClientSession *sess,
	__in	char *name,
	__out	char *data,
	__inout	int* len,
	__in	int flags
);

typedef int __stdcall SavePluginData(
	__in	ProtocolSession *psess,
	__in	ListenSession *lsess,
	__in	ClientSession *sess,
	__in	char *name,
	__in	char *data,
	__in	int len,
	__in	int flags
);

typedef ProtocolData* __stdcall FindProtocolDataStruct(
	__in	ProtocolSession *psess,
	__in	ProtocolData **data
);

//Control panel functions
typedef int __stdcall CloseProtocolSession(
	__in	ProtocolSession *psess
);

typedef int __stdcall CloseListenSession(
	__in	ProtocolSession *psess,
	__in	ListenSession *lsess
);

typedef int __stdcall CountListenSessions(
	__in	ProtocolSession psess
);

typedef void __stdcall ListenSessionIsCreated(
	__in	ListenSession lsess,
	__in	int flags
);

typedef void __stdcall ProtocolIsLoaded(
	__in	ProtocolSession psess
);

typedef InternalListenStructure* __stdcall GetCopyOfInternalListenStructure(
	__in	ProtocolSession *psess,
	__in	int id,
	__out	ListenSession *lsess
);

typedef InternalListenStructure* __stdcall GetCopyOfInternalListenStructureByLsess(
	__in	ProtocolSession *psess,
	__in	ListenSession *lsess
);

typedef InternalProtocolStructure* __stdcall GetCopyOfInternalProtocolStructures(
	__in	ProtocolSession psess
);

typedef int __stdcall RegisterListenSessionIsCreatedCallbackFunction(
	__in	ProtocolSession *psess,
	__in	ListenSessionIsCreated *func
);

typedef int __stdcall RegisterProtocolLoadedCallbackFunction(
	__in	ProtocolSession *psess,
	__in	ProtocolIsLoaded *func
);

typedef void __stdcall ReleaseCopyOfInternalListenStructure(
	__in	InternalListenStructure cpyInternalListenStructure
);

typedef void __stdcall ReleaseCopyOfInternalProtocolStructures(
	__in	InternalProtocolStructure cpyInternalProtocolstructure
);

//Multi column list functions

typedef int __stdcall CountListRows(
	__in	void *list
);

typedef int __stdcall GetListItem(
	__in	void *list,
	__in	int rowId,
	__in	int itemId,
	__out	char *data,
	__inout	int* len
);

typedef int __stdcall GetListItemSize(
	__in	void *list,
	__in	int rowId,
	__in	int itemId
);

typedef int __stdcall RemoveListRow(
	__in	void *list,
	__in	int rowId
);

typedef int __stdcall SetListItem(
	__in	void *list,
	__in	int rowId,
	__in	int itemId,
	__in	char *data,
	__in	int len
);

typedef int __stdcall AddListRow(
	__in	void *list,
	__in	Row *rows
);

typedef int __stdcall FindListItem(
	__in	void *list,
	__in	int columnId,
	__in	void *item,
	__in	int len,
	__in	int flags,
	__in	int startFromRow
);

//Time and date functions
typedef int __stdcall FormatDate(
	__in	int year,
	__in	int month,
	__in	int day
);

#ifdef _WIN64
typedef long long __stdcall FormatDateTime(
	__in	int year,
	__in	int month,
	__in	int day,
	__in	int hour,
	__in	int minutes,
	__in	int seconds
);
#else //32 bit
typedef int __stdcall FormatDateTime(
	__in	int year,
	__in	int month,
	__in	int day,
	__in	int hour,
	__in	int minutes,
	__in	int seconds,
	__out	long long *retval
);
#endif

typedef int __stdcall FormatTime(
	__in	int hour,
	__in	int minutes,
	__in	int seconds,
	__in	int milliseconds
);

typedef void RecoverDate(
	__in	int date,
	__out	TIMEDATE *odate
);

typedef void __stdcall RecoverDateTime(
	__in	long long sec,
	__out	TIMEDATE *date
);

typedef void __stdcall RecoverTime(
	__in	int time,
	__out	TIMEDATE *date
);

typedef struct Export_Table{
	int reserved;
	int size;
	GetProtocolNames *GetProtocolNames;
	StartProtocol *StartProtocol;
	BindToProtocolCallBack *BindToProtocolCallBack;
	StartListening *StartListening;
	HandleConnection *HandleConnection;
	RecvData *RecvData;
	SendData *SendData;
	CloseClientSession *CloseClientSession;
	GetClientSessionInfo * GetClientSessionInfo;
	PrepareCloseListenSession *PrepareCloseListenSession;
	CloseListenSession *CloseListenSession;
} Export_Table;

typedef void APIENTRY __asm_set_export_table(
	__in Export_Table* myExports
);

typedef struct APIImportTable{
	int reserved;
	int size;
	__asm_set_export_table* __asm_set_export_table;
	GetAPIVersion* GetAPIVersion;
	SetProtocolInformation* SetProtocolInformation;
	SetControlPanelOptions* SetControlPanelOptions;
	AcceptProtocol* AcceptProtocol;
	SavePluginData* SavePluginData;
	GetPluginData* GetPluginData;
	FindProtocolDataStruct* FindProtocolDataStruct;
	RequestToBindToNextProtocol* RequestToBindToNextProtocol;
	CallNextHandleConnection* CallNextHandleConnection;
	RecvData* RecvData;
	SendData* SendData;
	CloseClientSession* CloseClientSession;
	GetClientSessionInfo* GetClientSessionInfo;
	CloseListenSession* CloseListenSession;
	FormatDate* FormatDate;
	FormatTime* FormatTime;
	FormatDateTime* FormatDateTime;
	CountListRows* CountListRows;
	AddListRow* AddListRow;
	GetListItem* GetListItem;
	GetListItemSize* GetListItemSize;
	SetListItem* SetListItem;
	RemoveListRow* RemoveListRow;
	FindListItem* FindListItem;
	CloseProtocolSession* CloseProtocolSession;
	RecoverDateTime* RecoverDateTime;
	GetCopyOfInternalProtocolStructures* GetCopyOfInternalProtocolStructures;
	ReleaseCopyOfInternalProtocolStructures* ReleaseCopyOfInternalProtocolStructures;
	GetCopyOfInternalListenStructure* GetCopyOfInternalListenStructure;
	ReleaseCopyOfInternalListenStructure* ReleaseCopyOfInternalListenStructure;
	RegisterProtocolLoadedCallbackFunction* RegisterProtocolLoadedCallbackFunction;
	CountListenSessions* CountListenSessions;
	RegisterListenSessionIsCreatedCallbackFunction* RegisterListenSessionIsCreatedCallbackFunction;
	GetCopyOfInternalListenStructureByLsess* GetCopyOfInternalListenStructureByLsess;
	RecoverTime* RecoverTime;
	RecoverDate* RecoverDate;
	BindToProtocol* BindToProtocol;
} APIImportTable;

typedef void WINAPI __get_export_table(
	__out APIImportTable* oImport
);

void __stdcall __asm_exchange_tables(
	__in	__get_export_table *get_export_table_internal,
	__in	__get_export_table *get_export_table
);



__declspec(dllimport) char* __stdcall ConvertEscapedCharactersToBytes(
	__in	char *inputString,
	__in	int flags
);

__declspec(dllimport) int __stdcall DecodeURL(
	__in	char *inputString
);

__declspec(dllimport) void __stdcall DeleteDoubleSlashesInUrl(
	__in	char *inputString
);

__declspec(dllimport) int __stdcall EscapeAndSendValue(
	__in	ProtocolSession *psess,
	__in	ClientSession *sess,
	__in	char *inputString,
	__in	int timeout,
	__in	int flags
);

__declspec(dllimport) int __stdcall EscapeHtmlAndSendValue(
	__in	ProtocolSession *psess,
	__in	ClientSession *sess,
	__in	char *inputString,
	__in	int timeout,
	__in	int flags
);

__declspec(dllimport) void __stdcall FreeHTTPSession(
	__in	HTTPSession *httpSession
);

__declspec(dllimport) char* __stdcall GetCollectionItem(
	__in	void *coll,
	__in	char *itemName
);

__declspec(dllimport) HTTPSession* __stdcall RecvAndParseHTTP(
	__in	ProtocolSession *psess,
	__in	ClientSession *sess,
	__in	int maxHeaderSizeLimit,
	__in	int maxTimeToReceive,
	__in	int flags,
	__out	int *error
);

#ifdef _WIN64
	__declspec(dllimport) int __stdcall SendFile(
		__in	ProtocolSession *psess,
		__in	ClientSession *sess,
		__in	void *file,
		__in	int bufferSize,
		__in	long long start,
		__in	long long size,
		__in	int timeout
	);
#else
	__declspec(dllimport) int __stdcall SendFile(
		__in	ProtocolSession *psess,
		__in	ClientSession *sess,
		__in	void *file,
		__in	int bufferSize,
		__in	long long *start,
		__in	long long *size,
		__in	int timeout
	);
#endif

__declspec(dllimport) void __stdcall SendHTTPHeader(
	__in	ProtocolSession *psess,
	__in	ClientSession *sess,
	__in	ResponseHeader *resp,
	__in	int timeout
);

__declspec(dllimport) void __stdcall __asm_exchange_tables_http(
	__in	__get_export_table *get_export_table_internal,
	__in	__get_export_table *get_export_table
);

#endif